$(document).ready(function() {
	function getURLParameter(name) {
		return decodeURIComponent(
			(location.search.match(RegExp("[?|&]"+name+'=(.+?)(&|$)'))||[,null])[1]
		);  
	}
	
	var queue = function() {
		var myQueue = [];
		var isBusy = false;
		
		return {
			enqueue: function(item) {
				// must wrap an Ajax call within a function()
				//
				// we do it this way so that we can have
				// direct access to the success function
				// which we have to modify to call
				// queue.run()
				//
				// this also has the added benefit of making the
				// enqueue chains look a bit nicer
				//
				// this also means that you don't have to remember
				// to put queue.run() at the end of the success
				// callback function, which I tend to do
				myQueue.push(function() {
					//alert(item.name);
					$.ajax({
						url: item.url,
						data: item.data,
						dataType: "text",
						success: function(data) {
							//alert("success");
							item.success(data);
							isBusy = false;
							queue.run();
						},
						
						complete: function(xhr, status) {
							//alert("complete: "+status);
							
						},
						error: function(jqXHR, textStatus, errorThrow) {
							//alert("jqXHR: "+jqXHR+" textStatus: "+textStatus+" errorThrown: "+errorThrown);
						}
						
					});
				});
				
				// try running the queue
				// if we aren't in the middle of an ajax call
				// then we should be able to run an ajax job
				//
				// this is for when the queue is empty, and we add
				// more jobs later
				
				queue.run();
				
				return this;
			},
			
			// already a wrapped function
//			enqueueWrapped: function(item) {
//				myQueue.push(function() {
//					item();
//					queue.run();
//				});
//				return this;
//				
//			},
			
			run: function() {
				
				if (myQueue.length > 0 && !isBusy) {
					isBusy = true;
					myQueue.shift()();
				}
			}
		}
	}();
	
	var spinner = 
		$('<div></div>')
		.addClass('spinner')
		.append($('<img></img>').attr('src','/hal/commands/getFile/web/icons/ajax-loader.gif'));

	var mainDivs = [];
	mainDivs.push($('#runSettings > div'));
	mainDivs.push($('#runStatus > div'));
	mainDivs.push($('#ablationRounds > div'));
	mainDivs.push($('#ablationPlot > div'));
	mainDivs.push($('#subruns > div'));

	for(var i in mainDivs) {
		mainDivs[i].html(spinner.clone());
	}
	
	var pageRunId = getURLParameter('runId');
	var timer;
	var shouldAutorefresh = false;
	var ajaxInProgress = false;
	
	queue
	.enqueue({
		name: 'metaAlgorithmRunSettings',
		url: '/hal/getFormSnippet/runStatusMonitoring/metaAlgorithmRunSettings',
		data: { runId: pageRunId },
		success: function(data) {
			$('#runSettings > div').html(data);
		}
	})
	.enqueue({
		name: 'metaAlgorithmRunStatus',
		url: '/hal/getFormSnippet/runStatusMonitoring/metaAlgorithmRunStatus',
		data: { runId: pageRunId },
		success: function(data) {
			$('#runStatus > div').html(data);
		}
	})
	.enqueue({
		name: 'metaAlgorithmRunAblationRounds',
		url: '/hal/getFormSnippet/runStatusMonitoring/metaAlgorithmRunAblationRounds',
		data: { runId: pageRunId },
		success: function(data) {
			$('#ablationRounds > div').html(data);
		}
	})
	.enqueue({
		name: 'createPlot',
		url: '/hal/getFormSnippet/runStatusMonitoring/createPlot',
		data: { runId : pageRunId, plotType : 'ablationPlot' },
		success: function(data) {
			$('#ablationPlot > div').html(data);
		}
	});
	
	queue.run();
	
	var subruns = addSubrunsTableFunctionality(pageRunId);
	
	var runStatusAutoRefreshFunction = function() {
		queue
		.enqueue({
			name: 'metaAlgorithmRunStatus',
			url: '/hal/getFormSnippet/runStatusMonitoring/metaAlgorithmRunStatus',
			data: { runId: pageRunId },
			success: function(data) {
				$('#runStatus > div').html(data);
			}
		})
		.enqueue({
			name: 'createPlot',
			url: '/hal/getFormSnippet/runStatusMonitoring/createPlot',
			data: { runId : pageRunId, plotType : 'ablationPlot' },
			success: function(data) {
				$('#ablationPlot > div').html(data);
			}
		});
		
	}
	
	var runStatusRefresh = setInterval(runStatusAutoRefreshFunction,5000);

});