var createExperimentObject = function(data) {
	var separator = '_xXxversionxXx_';
	
	return {
		name: function() { return data.name+' (Copy of)'; },
		description: function() { return ''; },
		metaAlgorithm: function() { return data.implementation.name+separator+data.implementation.version; },
		instanceDistribution: function() { return data.instance.instances.name; },
		metric : function() { return data.instance.metric.name; },
		scenarioOptions : function() { return data.scenario; },
		instanceOptions : function() { return data.instance.options; },
		configurationOptions : function() { return undefined; },
		environment : function() { return data.environment.name; },
//		targetAlgorithm: function() { 
//			var array = new Array();
//			
//			array.push(data.instance.algorithms.implementation.name+separator+data.instance.algorithms.implementation.version+separator+data.instance.algorithms.name);
//			return array;
//		},
		
	}
}

var createAnalysisExperimentObject = function(data) {
	var separator = '_xXxversionxXx_';
	
	var genericExperimentObj = createExperimentObject(data);
	genericExperimentObj.targetAlgorithm = function() {
		var array = new Array();
		for(var i in data.instance.algorithms.algorithms) {
			var algorithm = data.instance.algorithms.algorithms[i];
			
			var algorithmName = algorithm.implementation.name+separator+algorithm.implementation.version;
			
			if (algorithm.name) { algorithmName = algorithmName+separator+algorithm.name; }
			else { algorithmName = algorithmName+separator+algorithm.implementation.name; }
			
			array.push(algorithmName);
		}
		return array;
	}
	
	return genericExperimentObj;
}

var createConfigurationExperimentObject = function(data) {
	var separator = '_xXxversionxXx_';
	
	var genericExperimentObj = createExperimentObject(data);
	genericExperimentObj.targetAlgorithm = function() {
		var array = new Array();
		
		array.push(data.instance.algorithms.implementation.name+separator+data.instance.algorithms.implementation.version+separator+data.instance.algorithms.name);
		return array;
	}
	
	genericExperimentObj.configurationOptions = function() { return undefined; }
	
	// for cases like ParamILS and old ROAR, these configuration experiments didn't have
	// configuration options, but SMAC now does, so we should pass them along.
	if (data.configuration) {
		genericExperimentObj.configurationOptions = function() {
			return data.configuration;
		}
	}
	
	genericExperimentObj.featureExtractors = function() { return undefined; }
	
	if (data.instance.extractors) {
		var array = [];
		
		for(var i=0; i<data.instance.extractors.length; i=i+1) {
			var extractor = data.instance.extractors[i];
			array.push(extractor.name+separator+extractor.version);
		}
		
		genericExperimentObj.featureExtractors = function() {
			if (array.length < 1) return undefined;
			return array;
		}
	}
	
	return genericExperimentObj;
}

HAL.createOrCloneMenu = function(options) {
	if ( !(this instanceof HAL.createOrCloneMenu) ) {
	      return new HAL.createOrCloneMenu(options);
	}
	
	var _headerContent = options.headerContent == null ? 'Experiment creation mode' : options.headerContent;
	var _idPrefix = options.idPrefix == null ? 'noid' : options.idPrefix;
	var _isMenuEnabled = options.enabled == null ? true : options.enabled;
	var _isMenuExpanded = options.expanded == null ? true : options.expanded;
	var _showPrevious = options.showPrevious == null ? true : options.showPrevious;
	var _showNext = options.showNext == null ? true : options.showNext;
	var _description = options.description ? options.description : 'Define a brand new experiment or clone an existing experiment';
	
	var _menu = new HAL.menu({
		header : _headerContent,
		enabled : _isMenuEnabled,
		expanded : _isMenuExpanded,
		showPrevious : _showPrevious,
		showNext : _showNext
	});

	// experiment information setup and navigation
	
	var description = $('<div></div>').addClass('formFieldDescription').html(_description);
	//.html('Define a brand new configuration experiment or clone an already defined experiment');
	
	var newExperimentButton = $('<input></input>').attr('type','button').attr('value','Define new experiment');
	
	var cloneExperimentButton = $('<input></input>').attr('type','button').attr('value','Clone an existing experiment');
	
	_menu.getContentElement().append(description).append(newExperimentButton).append(cloneExperimentButton);
	
	return {
		getMenuElement : function() {
			return _menu.getMenuElement();
		},
		getMenuObject : function() {
			return _menu;
		},
		getContentElement : function() {
			return _menu.getContentElement();
		},
		addNewButtonOnClickHandler : function(f) {
			newExperimentButton.click(f);
			return this;
		},
		addCloneButtonOnClickHandler : function(f) {
			cloneExperimentButton.click(f);
			return this;
		},
		addOnNext : function(f) {
			_menu.addOnNext(f);
			return this;
		},
		addOnPrevious : function(f) {
			_menu.addOnPrevious(f);
			return this;
		},
		clear : function() {
			return this;
		},
		disable : function() {
			_menu.disable();
			return this;
		},
		enable : function() {
			_menu.enable();
			return this;
		},
		expand : function() {
			_menu.expand();
			return this;
		},
		collapse : function() {
			_menu.collapse();
			return this;
		}
	};
};

HAL.cloneConfigurationExperimentSelectionMenu = function(options) {
	if ( !(this instanceof HAL.cloneConfigurationExperimentSelectionMenu) ) {
	      return new HAL.cloneConfigurationExperimentSelectionMenu(options);
	}
	
	var _headerContent = options.headerContent == null ? 'Clone experiment mode' : options.headerContent;
	var _idPrefix = options.idPrefix == null ? 'noid' : options.idPrefix;
	var _isMenuEnabled = options.enabled == null ? true : options.enabled;
	var _isMenuExpanded = options.expanded == null ? true : options.expanded;
	var _showPrevious = options.showPrevious == null ? true : options.showPrevious;
	var _showNext = options.showNext == null ? true : options.showNext;
	
	var _menu = new HAL.menu({
		header : _headerContent,
		enabled : _isMenuEnabled,
		expanded : _isMenuExpanded,
		showPrevious : _showPrevious,
		showNext : _showNext
	});

	// experiment information setup and navigation
	
	var description = $('<div></div>').addClass('formFieldDescription').html('Choose an experiment to clone.');
	
	var spinnerDiv = $('<div></div>').addClass('spinner').html($('<img></img')
			.attr('src', '/hal/commands/getFile/web/icons/ajax-loader.gif').attr('alt', 'Loading. Please wait.'));
	
	var selector = $('<select></select>');
	
	//var nullOption = $('<option></option>').html('Loading ...').attr('value','').appendTo(selector);
	
	_menu.disableNextButton();
	
	_menu.setBusy(true);
	
	$.ajax({
		//url: '/jsp/getAlgorithmRunNames.jsp',
		url: '/hal/cloneAlgorithm/getAlgorithmRunNames',
		data: { "filter":"configuration" },
		dataType: 'json',
		success: function(data) {
			if (!data.successful) {
				// some error message
			} else {
				
				
				_menu.setBusy(false);
				//nullOption.remove();
				
//				spinnerDiv.remove();
				
				//nullOption = $('<option></option>').html('').attr('value','').appendTo(selector);
				
				
				for(var i in data.result) {
					var experiment = data.result[i];
					$('<option></option>').html(experiment.name).attr('value',experiment.id).appendTo(selector);
					
				}
				
				_menu.getContentElement().append(selector);
				
				_menu.enableNextButton();
				
				
			}
		}
	});
	
	_menu.getContentElement().append(description);
	
	return {
		getMenuElement : function() {
			return _menu.getMenuElement();
		},
		getMenuObject : function() {
			return _menu;
		},
		getContentElement : function() {
			return _menu.getContentElement();
		},
		addSelectorOnClick : function(f) {
			selector.click(f);
			return this;
		},
		getSelectorValue : function() {
			return selector.val();
		},
		clear : function() {
			selector.val('');
			return this;
		},
		addOnNext : function(f) {
			_menu.addOnNext(f);
			return this;
		},
		addOnPrevious : function(f) {
			_menu.addOnPrevious(f);
			return this;
		},
		disable : function() {
			_menu.disable();
			return this;
		},
		enable : function() {
			_menu.enable();
			return this;
		},
		expand : function() {
			_menu.expand();
			return this;
		},
		collapse : function() {
			_menu.collapse();
			return this;
		},
		isLoading : function() {
			_menu.setBusy(true);
			return this;
		},
		hasLoaded : function() {
			_menu.setBusy(false);
			return this;
		}
	};
};

HAL.cloneAnalysisExperimentSelectionMenu = function(options) {
	if ( !(this instanceof HAL.cloneAnalysisExperimentSelectionMenu) ) {
	      return new HAL.cloneAnalysisExperimentSelectionMenu(options);
	}
	
	var _headerContent = options.headerContent == null ? 'Clone experiment mode' : options.headerContent;
	var _idPrefix = options.idPrefix == null ? 'noid' : options.idPrefix;
	var _isMenuEnabled = options.enabled == null ? true : options.enabled;
	var _isMenuExpanded = options.expanded == null ? true : options.expanded;
	var _showPrevious = options.showPrevious == null ? true : options.showPrevious;
	var _showNext = options.showNext == null ? true : options.showNext;
	
	var _menu = new HAL.menu({
		header : _headerContent,
		enabled : _isMenuEnabled,
		expanded : _isMenuExpanded,
		showPrevious : _showPrevious,
		showNext : _showNext
	});

	// experiment information setup and navigation
	
	var description = $('<div></div>').addClass('formFieldDescription').html('Choose an experiment to clone.');
	
	var spinnerDiv = $('<div></div>').addClass('spinner').html($('<img></img')
			.attr('src', '/hal/commands/getFile/web/icons/ajax-loader.gif').attr('alt', 'Loading. Please wait.'));
	
	var selector = $('<select></select>');
	
	//var nullOption = $('<option></option>').html('Loading ...').attr('value','').appendTo(selector);
	
	//_menu.disableNextButton();
	
	_menu.setBusy(true);
	
	$.ajax({
		//url: '/jsp/getAlgorithmRunNames.jsp',
		url: '/hal/cloneAlgorithm/getAlgorithmRunNames',
		dataType: 'json',
		data : { filter: 'analysis' },
		success: function(data) {
			_menu.clearMessages();
			
			if (!data.successful) {
				// some error message
				
				for(var i in data.errorMessages) {
					_menu.addErrorMessage(data.errorMessages[i]);
				}
			} else {
				
				//nullOption.remove();
				
//				spinnerDiv.remove();
				
				_menu.setBusy(false);
				
				//nullOption = $('<option></option>').html('').attr('value','').appendTo(selector);
				
				
				for(var i in data.result) {
					var experiment = data.result[i];
					$('<option></option>').html(experiment.name).attr('value',experiment.id).appendTo(selector);
					
				}
				
				_menu.getContentElement().append(selector);
				
				//_menu.enableNextButton();
				
			}
		}
	});
	
	_menu.getContentElement().append(description);
	
	return {
		getMenuElement : function() {
			return _menu.getMenuElement();
		},
		getMenuObject : function() {
			return _menu;
		},
		getContentElement : function() {
			return _menu.getContentElement();
		},
		addSelectorOnClick : function(f) {
			selector.click(f);
			return this;
		},
		getSelectorValue : function() {
			return selector.val();
		},
		clear : function() {
			selector.val('');
			return this;
		},
		addOnNext : function(f) {
			_menu.addOnNext(f);
			return this;
		},
		addOnPrevious : function(f) {
			_menu.addOnPrevious(f);
			return this;
		},
		disable : function() {
			_menu.disable();
			return this;
		},
		enable : function() {
			_menu.enable();
			return this;
		},
		expand : function() {
			_menu.expand();
			return this;
		},
		collapse : function() {
			_menu.collapse();
			return this;
		}
	};
};

