HAL.metaAlgorithmPluginSelector = function(options) {
	if ( !(this instanceof HAL.metaAlgorithmPluginSelector) ) {
	      return new HAL.metaAlgorithmPluginSelector(options);
	}
	
	var initialValue = options.initialValue;
	
	var _metaAlgorithmPluginLabel = $('<label></label>');
	_metaAlgorithmPluginLabel.attr('for', options.idPrefix+'_metaAlgorithmPluginName');
	_metaAlgorithmPluginLabel.html('Meta-algorithmic procedure');
	
	var _metaAlgorithmPluginField = $('<select></select>');
	_metaAlgorithmPluginField.attr('name', options.idPrefix+'_metaAlgorithmPluginName');
	_metaAlgorithmPluginField.attr('id', options.idPrefix+'_metaAlgorithmPluginName');
	_metaAlgorithmPluginField.append($('<option></option>').attr('value', '').html('Populating...'));
	
	var _metaAlgorithmPluginContainer = $('<div></div>');
	_metaAlgorithmPluginContainer.addClass('formElement');
	if (options.showLabel == null || options.showLabel) {
		_metaAlgorithmPluginContainer.append(_metaAlgorithmPluginLabel);
	}
	_metaAlgorithmPluginContainer.append(_metaAlgorithmPluginField);
	
	var _changeCallbacks = [];
	_metaAlgorithmPluginField.change(function() {
		for (var i in _changeCallbacks) {
			_changeCallbacks[i]();
		}
	});
	
	$.post('/hal/getFormSnippet/utilities/getDefinedMetaAlgorithmPlugins',
		{
			'hal_httpResponseContentType' : 'json',
			'compatibilityFilter' : options.compatibilityFilter
		},
		function(response) {
			if (response.successful) {
				var plugins = response.result.metaAlgorithmPlugins;
				_metaAlgorithmPluginField.empty();

				var defaultOpt = $('<option></option>');
				defaultOpt.attr('value', '');
					
				if (numberOfProperties(plugins) !== 0) {
					defaultOpt.html('--- Select a meta-algorithmic procedure ---');
					_metaAlgorithmPluginField.append(defaultOpt);
					
					for (var i in plugins) {
						var val = plugins[i].name+'_xXxversionxXx_'+plugins[i].version;
						
						var id = plugins[i].name;
						if (plugins[i].version !== null && plugins[i] !== '') {
							id = id+' ('+plugins[i].version+')';
						}
						
						var opt = $('<option></option>');
						opt.attr('value', val);
						opt.html(id);
	
						_metaAlgorithmPluginField.append(opt);
					}
					
					if (initialValue) {
						_metaAlgorithmPluginField.children().each(function(index,item) {
							if ($(item).val() == initialValue) { _metaAlgorithmPluginField.val(initialValue); }
						});
					}
					
				} else {
					defaultOpt.html('--- No compatible procedures registered! ---');
					_metaAlgorithmPluginField.append(defaultOpt);
				}
			}
		},
		'json'
	);
	
	return {
		getElement : function() {
			return _metaAlgorithmPluginContainer;
		},
		addChangeCallback : function(callback) {
			_changeCallbacks.push(callback);
		},
		getValue : function() {
			return _metaAlgorithmPluginField.val();
		},
		clear : function() {
			return this.setValue('');
		},
		setValue : function(value) {
			_metaAlgorithmPluginField.children().each(function(index,item) {
				if ($(item).val() == value) { _metaAlgorithmPluginField.val(value); }
			});
			return this;
		}
	};
};

HAL.defaultExperimentInformationMenu = function(options) {
	if ( !(this instanceof HAL.defaultExperimentInformationMenu) ) {
	      return new HAL.defaultExperimentInformationMenu(options);
	}
	
	var _headerContent = options.headerContent == null ? 'Experiment information' : options.headerContent;
	var _idPrefix = options.idPrefix == null ? 'noid' : options.idPrefix;
	var _isMenuEnabled = options.enabled == null ? true : options.enabled;
	var _isMenuExpanded = options.expanded == null ? true : options.expanded;
	var _showPrevious = options.showPrevious == null ? true : options.showPrevious;
	var _showNext = options.showNext == null ? true : options.showNext;
	var _metaAlgorithmicPluginCompatibilityFilter = options.metaAlgorithmicPluginCompatibilityFilter == null ? '' : options.metaAlgorithmicPluginCompatibilityFilter;
	
	var spinnerDiv = $('<div></div>').addClass('spinner').html($('<img></img')
			.attr('src', '/hal/commands/getFile/web/icons/ajax-loader.gif').attr('alt', 'Loading. Please wait.'));
	
	var _menu = new HAL.menu({
		header : _headerContent,
		enabled : _isMenuEnabled,
		expanded : _isMenuExpanded,
		showPrevious : _showPrevious,
		showNext : _showNext
	});

	// experiment information setup and navigation
	
	var _experimentNameField = new HAL.textField({
		id : _idPrefix+'_experimentName',
		label : 'Experiment name',
		initialValue: (options.defaults) ? options.defaults.experimentName : undefined
	});
	_menu.getContentElement().append(_experimentNameField.getElement());
	
	var _experimentNotesField = new HAL.textArea({
		id : _idPrefix+'_experimentNotes',
		label : 'Experiment description/notes (optional)',
		intialValue : (options.defaults) ? options.defaults.experimentDescription : undefined
	});
	_menu.getContentElement().append(_experimentNotesField.getElement());
		
	var _metaAlgorithmicProcedureField = new HAL.metaAlgorithmPluginSelector({
		idPrefix : _idPrefix,
		compatibilityFilter : _metaAlgorithmicPluginCompatibilityFilter,
		initialValue: (options.defaults) ? options.defaults.metaAlgorithm : undefined
	});
	
	_menu.getContentElement().append(_metaAlgorithmicProcedureField.getElement());

	var _instanceDistributionField = new HAL.instanceDistributionSelector({
		idPrefix : _idPrefix,
		initialValue : (options.defaults) ? options.defaults.instanceDistribution : undefined
	});
	
	_menu.getContentElement().append(_instanceDistributionField.getElement());
	
	var menuContentChildren;
	
	return {
		
		getMenuElement : function() {
			return _menu.getMenuElement();
		},
		getMenuObject : function() {
			return _menu;
		},
		getContentElement : function() {
			return _menu.getContentElement();
		},
		getExperimentName : function() {
			return _experimentNameField.getValue();
		},
		getExperimentNotes : function() {
			return _experimentNotesField.getValue();
		},
		getMetaAlgorithmicProcedure : function() {
			return _metaAlgorithmicProcedureField.getValue();
		},
		getInstanceDistribution : function() {
			return _instanceDistributionField.getValue();
		},
		validate : function() {
			if (_metaAlgorithmicProcedureField.getValue() !== '' &&
					_instanceDistributionField.getValue() !== '' &&
					_experimentNameField.getValue() !== '') {
				return true;
			} else {
				return false;
			}
		},
		addOnNext : function(f) {
			_menu.addOnNext(f);
			return this;
		},
		addOnPrevious : function(f) {
			_menu.addOnPrevious(f);
			return this;
		},
		isLoading : function() {
			_menu.setBusy(true);
			return this;
//			menuContentChildren = _menu.getContentElement().children();
//			menuContentChildren.detach();
//			_menu.getContentElement().append(spinnerDiv);
//			_menu.disableNextButton();
//			return this;
		},
		hasLoaded : function() {
			_menu.setBusy(false);
			return this;
//			_menu.getContentElement().children().remove();
//			_menu.getContentElement().append(menuContentChildren);
//			_menu.enableNextButton();
//			return this;
		},
		clear : function() {
			_experimentNameField.clear();
			_experimentNotesField.clear();
			_metaAlgorithmicProcedureField.clear();
			_instanceDistributionField.clear();
			return this;
		},
		disable : function() {
			_menu.disable();
			return this;
		},
		enable : function() {
			_menu.enable();
			return this;
		},
		expand : function() {
			_menu.expand();
			return this;
		},
		collapse : function() {
			_menu.collapse();
			return this;
		},
		setDefaults : function(defaults) {
			_experimentNameField.setValue(defaults.experimentName);
			_experimentNotesField.setValue(defaults.experimentDescription);
			_metaAlgorithmicProcedureField.setValue(defaults.metaAlgorithm);
			_instanceDistributionField.setValue(defaults.instanceDistribution);
			
			return this;
			
		}
	};
};

HAL.defaultMetaAlgorithmParametersMenu = function(options) {
	if ( !(this instanceof HAL.defaultMetaAlgorithmParametersMenu) ) {
	      return new HAL.defaultMetaAlgorithmParametersMenu(options);
	}
	
	var _headerContent = options.headerContent == null ? 'Experiment parameters' : options.headerContent;
	var _idPrefix = options.idPrefix == null ? 'noid' : options.idPrefix;
	var _isMenuEnabled = options.enabled == null ? true : options.enabled;
	var _isMenuExpanded = options.expanded == null ? true : options.expanded;
	var _fillWithDefaults = options.fillWithDefaults == null ? true : options.fillWithDefaults;
	var _showPrevious = options.showPrevious == null ? true : options.showPrevious;
	var _showNext = options.showNext == null ? true : options.showNext;
	var _algorithm = options.algorithm == null ? null : options.algorithm;
	
	var _menu = new HAL.menu({
		header : _headerContent,
		enabled : _isMenuEnabled,
		expanded : _isMenuExpanded,
		showPrevious : _showPrevious,
		showNext : _showNext
	});
	
	var _designProcedureScenarioParametersMenu = new HAL.menu({
		header : 'Scenario settings',
		enabled : true,
		expanded : false,
		showPrevious : false,
		showNext : false
	});
	
	var _designProcedureConfigurationParametersMenu = new HAL.menu({
		header : 'Configuration settings',
		enabled : true,
		expanded : false,
		showPrevious : false,
		showNext : false
	});
	
	var _designProcedureInstanceOptionParametersMenu = new HAL.menu({
		header : 'Other settings',
		enabled : true,
		expanded : false,
		showPrevious : false,
		showNext : false
	});
	
	// metric
	var _metricSelectionField = new HAL.metricSelector({
		idPrefix : _idPrefix+'_metric',
		initialValue : (options.defaults) ? options.defaults.metric : undefined
	});
	
	_menu.getContentElement().append(_metricSelectionField.getElement());
	_menu.getContentElement().append(_designProcedureConfigurationParametersMenu.getMenuElement());
	_menu.getContentElement().append(_designProcedureScenarioParametersMenu.getMenuElement());
	_menu.getContentElement().append(_designProcedureInstanceOptionParametersMenu.getMenuElement());
	
	var _configurationParameterSettingsField = new HAL.parameterSettingsField({
		idPrefix : _idPrefix+'_configurationSpace',
		fillWithDefaults : _fillWithDefaults,
		parameterSpace : {}
	});
	var _scenarioParameterSettingsField = new HAL.parameterSettingsField({
		idPrefix : _idPrefix+'_scenarioSpace',
		fillWithDefaults : _fillWithDefaults,
		parameterSpace : {}
	});
	var _supportedInstanceOptionParameterSettingsField = new HAL.parameterSettingsField({
		idPrefix : _idPrefix+'_supportedInstanceOptionSpace',
		fillWithDefaults : _fillWithDefaults,
		parameterSpace : {}
	});
	
	_designProcedureConfigurationParametersMenu.getContentElement().append(_configurationParameterSettingsField.getElement());
	_designProcedureScenarioParametersMenu.getContentElement().append(_scenarioParameterSettingsField.getElement());
	_designProcedureInstanceOptionParametersMenu.getContentElement().append(_supportedInstanceOptionParameterSettingsField.getElement());
		
	var _refresh = function(filterOptions, successCallback, failureCallback, defaults) {
		if (filterOptions.sourceAlgorithm == null) {
			return;
		}
		
		_menu.setBusy(true);
		
		_algorithm = filterOptions.sourceAlgorithm;
		
		var doneWithSpaces = false;
		var doneWithMetrics = false;
		
		// if neither options.defaults nor defaults are defined then undefined
		// otherwise one of them must be defined so we choose the one that is.
		var spaceDefaults = (options.defaults && defaults) ? undefined : (options.defaults) ? options.defaults : defaults;
		
		$.post('/hal/getFormSnippet/utilities/getAlgorithmParameterSpaces',
			{
				hal_httpResponseContentType : 'json',
				algorithm : _algorithm
			},
			function(response) {
				if (response.successful) {
					var parameterSpaces = response.result.parameterSpaces;
					
					var scenarioSpace = parameterSpaces.scenarioSpace;
					var configurationSpace = parameterSpaces.configurationSpace;
					var instanceOptionSpace = parameterSpaces.supportedInstanceOptionSpace;
					
					// if we have defaults, we edit the domain objects returned by the ajax call
					// with new defaults
					
					if (spaceDefaults) {
						var scenarioDefaults = spaceDefaults.scenario;
						
						if (scenarioDefaults) {
							for(var i in scenarioSpace) { // actual names since this is an object
								if (scenarioDefaults[i]) {
									scenarioSpace[i].domain.default = scenarioDefaults[i];
								}
							}
						}
						
						var instanceDefaults = spaceDefaults.instance;
						
						if (instanceDefaults) {
							for(var i in instanceOptionSpace) {
								if (instanceDefaults[i]) {
									instanceOptionSpace[i].domain.default = instanceDefaults[i];
								}
							}
						}
						
						var configurationDefaults = spaceDefaults.configuration;
						
						if (configurationDefaults) {
							for(var i in configurationSpace) {
								if (configurationDefaults[i]) {
									configurationSpace[i].domain.default = configurationDefaults[i];
								}
							}
						}
					}
					
					_designProcedureConfigurationParametersMenu.disable();
					_designProcedureScenarioParametersMenu.disable();
					_designProcedureInstanceOptionParametersMenu.disable();
					
					_configurationParameterSettingsField.updateParameterSpace(configurationSpace);
					_scenarioParameterSettingsField.updateParameterSpace(scenarioSpace);
					_supportedInstanceOptionParameterSettingsField.updateParameterSpace(instanceOptionSpace);
					
					_designProcedureConfigurationParametersMenu.setHeaderContent('Configuration settings ('+_configurationParameterSettingsField.getNumberOfParameters()+')');
					_designProcedureScenarioParametersMenu.setHeaderContent('Scenario settings ('+_scenarioParameterSettingsField.getNumberOfParameters()+')');
					_designProcedureInstanceOptionParametersMenu.setHeaderContent('Other settings ('+_supportedInstanceOptionParameterSettingsField.getNumberOfParameters()+')');
					
					if (_configurationParameterSettingsField.getNumberOfParameters() > 0) {
						_designProcedureConfigurationParametersMenu.enable();
					}
					if (_scenarioParameterSettingsField.getNumberOfParameters() > 0) {
						_designProcedureScenarioParametersMenu.enable();
					}
					if (_supportedInstanceOptionParameterSettingsField.getNumberOfParameters() > 0) {
						_designProcedureInstanceOptionParametersMenu.enable();
					}
				} else {
					failureCallback();
				}
				
				doneWithSpaces = true;
				
				if (doneWithMetrics) {
					successCallback();
					_menu.setBusy(false);
				}
			},
			'json'
		);
		
		$.post('/hal/getFormSnippet/utilities/getDefinedMetrics',
			{
				hal_httpResponseContentType : 'json'
			},
			function(response) {
				if (response.successful) {
					var metrics = response.result.metrics;
					
					_metricSelectionField.updateMetricList(metrics);
				} else {
					failureCallback();
				}
				
				doneWithMetrics = true;
				
				if (doneWithSpaces) {
					successCallback();
					_menu.setBusy(false);
				}
			},
			'json'
		);
	};
	
	_refresh(
		{
			sourceAlgorithm : _algorithm
		},
		function() {},
		function() {}
	);
	
	return {
		getMenuElement : function() {
			return _menu.getMenuElement();
		},
		getMenuObject : function() {
			return _menu;
		},
		getContentElement : function() {
			return _menu.getContentElement();
		},
		setDefaults : function(defaults) {
			_metricSelectionField.setValue(defaults.metric);
		},
		enable : function() {
			_menu.enable();
		},
		disable : function() {
			_menu.disable();
		},
		expand : function() {
			_menu.expand();
		},
		collapse : function() {
			_menu.collapse();
		},
		setBusy : function(isBusy) {
			_menu.setBusy(isBusy);
		},
		addOnNext : function(f) {
			_menu.addOnNext(f);
		},
		addOnPrevious : function(f) {
			_menu.addOnPrevious(f);
		},
		update : function(filterOptions, successCallback, failureCallback, defaults) {
			_refresh(filterOptions, successCallback, failureCallback, defaults);
		}
	};
};