HAL.multipleAlgorithmAnalysisForm = function(options) {
	if ( !(this instanceof HAL.multipleAlgorithmAnalysisForm) ) {
	      return new HAL.multipleAlgorithmAnalysisForm(options);
	}
	
	var _idPrefix = options.idPrefix == null ? 'newMultipleAlgorithmAnalysis' : options.idPrefix;
	
	var _form = $('<form></form>');
	_form.attr('id', _idPrefix+'_form');
	
	_menuItems = new Array();
	
	var _newCreateOrCloneMenu = new HAL.createOrCloneMenu({
		idPrefix : _idPrefix,
		enabled : true,
		expanded : true,
		showPrevious : false,
		showNext : false,
		description: 'Define a brand new analysis experiment or clone an already defined experiment'
	}).addNewButtonOnClickHandler(function() {
		_newCreateOrCloneMenu.collapse().disable();
		_experimentInformationMenu.enable().expand();
		
		_newExperimentButtonHandler();
		
	}).addCloneButtonOnClickHandler(function() {
		_newCreateOrCloneMenu.collapse().disable();
		_cloneExperimentMenu.enable().expand();
	});
	
	_menuItems.push(_newCreateOrCloneMenu);
	
	var _cloneExperimentMenu = new HAL.cloneAnalysisExperimentSelectionMenu({
		idPrefix : _idPrefix,
		showPrevious : false,
		showNext : true,
		expanded: false,
		enabled: false,
	}).addOnPrevious(function() {
		_cloneExperimentMenu.collapse().disable();
		_newCreateOrCloneMenu.enable().expand();
	
	}).addOnNext(function() {
		_cloneExperimentMenu.collapse().disable();
		_experimentInformationMenu.enable().expand().isLoading();
		
		_cloneExperimentNextButtonHandler();
	});
	
	_menuItems.push(_cloneExperimentMenu);
	
	var _experimentInformationMenu = new HAL.defaultExperimentInformationMenu({
		idPrefix : _idPrefix,
		metaAlgorithmicPluginCompatibilityFilter : 'MultipleAlgorithmAnalysis',
		expanded: false,
		enabled : false,
		showPrevious : false,
		showNext : true
	});
	
	_menuItems.push(_experimentInformationMenu);
	
	var _targetAlgorithmMenu = new HAL.defaultMultipleAlgorithmSelectionMenu({
		idPrefix : _idPrefix,
		enabled : false,
		expanded : false,
		minimumAlgorithms : 1,
		maximumAlgorithms : null,
		includeParameterisedAlgorithms : true,
		includeParameterlessAlgorithms : true,
		forceParameterlessSelection : true
	});
	
	_menuItems.push(_targetAlgorithmMenu);
	
	var _parametersMenu = new HAL.defaultMetaAlgorithmParametersMenu({
		idPrefix : _idPrefix,
		headerContent : 'Analysis procedure parameters',
		enabled : false,
		expanded : false
	});
	
	_menuItems.push(_parametersMenu);
	
	var _executionEnvironmentMenu = new HAL.defaultExecutionEnvironmentMenu({
		idPrefix : _idPrefix,
		enabled : false,
		expanded : false,
		showNext : false
	});
	
	_menuItems.push(_executionEnvironmentMenu);

	// CLONE BUTTON HANDLER FUNCTION
	
	var _cloneExperimentNextButtonHandler = function() {
		$.ajax({
			//url: '/jsp/getAlgorithmRunRequestJSON.jsp',
			url: '/hal/cloneAlgorithm/getAlgorithmRunDetails',
			data: { runId : _cloneExperimentMenu.getSelectorValue() },
			dataType: 'json',
			success: function(data) {
				if (!data.successful) {
					// error message
					for(var i in data.errorMessages) {
						_experimentInformationMenu.addErrorMessage(data.errorMessages[i]);
					}
					
				} else {
					var experimentObj = createAnalysisExperimentObject(data.result);
					registerButtonHandlerFunctions({
						name : experimentObj.name(),
						description : experimentObj.description(),
						metaAlgorithm : experimentObj.metaAlgorithm(),
						instanceDistribution : experimentObj.instanceDistribution(),
						environment : experimentObj.environment(),
						metric : experimentObj.metric(),
						targetAlgorithm : experimentObj.targetAlgorithm(),
						scenarioOptions : experimentObj.scenarioOptions(),
						instanceOptions : experimentObj.instanceOptions(),
						configurationOptions : experimentObj.configurationOptions(),
					})
				}
			}
		});
	}
	
	// NEW BUTTON HANDLER FUNCTION
	
	var _newExperimentButtonHandler = function() {
		_experimentInformationMenu.getMenuObject().hidePreviousButton();
		
		registerButtonHandlerFunctions();
	}
	
	// ADD CLICK HANDLERS TO VARIOUS BUTTONS SECTION

	var registerButtonHandlerFunctions = function(defaults) {
		
		if (defaults) {
			submitButton.attr('disabled',false);
			
			_experimentInformationMenu.hasLoaded().setDefaults({
				experimentName : defaults.name,
				experimentDescription : defaults.description,
				metaAlgorithm : defaults.metaAlgorithm,
				instanceDistribution : defaults.instanceDistribution,
				
			});
			
			_executionEnvironmentMenu.setDefaults({ environment : defaults.environment });
			
		}

		_experimentInformationMenu.addOnPrevious(function() {
			_cloneExperimentMenu.enable().expand();
			_experimentInformationMenu.collapse().disable();
		});
		
		_experimentInformationMenu.addOnNext(function() {
			if (_experimentInformationMenu.validate()) {
				_experimentInformationMenu.disable();
				
				_targetAlgorithmMenu.enable();
				_targetAlgorithmMenu.expand();
					
				_targetAlgorithmMenu.update(
					{
						'compatibleWithInstanceDistribution' : _experimentInformationMenu.getInstanceDistribution()	
					},
					function() {
						if (defaults) {
							_targetAlgorithmMenu.setDefaults({ targetAlgorithms : defaults.targetAlgorithm });
						}
						
					},
					function() {}
				);
			}
		});
		
		_targetAlgorithmMenu.addOnPrevious(function() {
			_targetAlgorithmMenu.disable();
			
			_experimentInformationMenu.enable().expand();
		});
		
		_targetAlgorithmMenu.addOnNext(function() {
			_targetAlgorithmMenu.disable();
			_parametersMenu.enable();
			_parametersMenu.expand();
				
			_parametersMenu.update(
				{
					sourceAlgorithm : _experimentInformationMenu.getMetaAlgorithmicProcedure()
				},
				function() {
					if(defaults) {
						_parametersMenu.setDefaults({ metric : defaults.metric });
					}
				},
				function() {},
				(!defaults) ? undefined : {
					scenario : defaults.scenarioOptions,
					instance : defaults.instanceOptions,
					configuration : defaults.configurationOptions,
				}
			);
		});
		
		_parametersMenu.addOnPrevious(function() {
			_parametersMenu.disable();
			_targetAlgorithmMenu.enable();
			_targetAlgorithmMenu.expand();
		});
		
		_parametersMenu.addOnNext(function() {
			_parametersMenu.disable();
			_executionEnvironmentMenu.enable();
			_executionEnvironmentMenu.expand();
		});
		
		_executionEnvironmentMenu.addOnPrevious(function() {
			_executionEnvironmentMenu.clearMessages();
			_executionEnvironmentMenu.disable();
			
			_parametersMenu.enable();
			_parametersMenu.expand();
		});
		
		
	}
	
	var submitButton = $('<input></input>')
	.attr({ type: 'button', value: 'Submit', disabled: true })
	.click(function() {
		_executionEnvironmentMenu.clearMessages();
		submitButton.attr('disabled', true);
		
		var formData = _form.serialize();
		formData = formData+'&hal_httpResponseContentType=json';
		
		$.post('/hal/processFormSnippet/createMultipleAlgorithmAnalysis/createMultipleAlgorithmAnalysis',
			formData,
			function(response) {
				submitButton.attr('disabled', false);
				
				if (response.successful) {
					_executionEnvironmentMenu.addSuccessMessage('Successfully created algorithm analysis experiment!');
					_executionEnvironmentMenu.addSuccessMessage("Redirecting back to HAL's homepage");
					setTimeout(function() {
						window.location = "/hal";
					}, 2000)
				} else {
					for (var i in response.errorMessages) {
						_executionEnvironmentMenu.addErrorMessage(response.errorMessages[i]);
					}
				}
			},
			'json'
		);
	});
	
	_executionEnvironmentMenu.addNavigationElement(submitButton);
	_executionEnvironmentMenu.addEnvironmentChangeCallback(function() {
		if (_executionEnvironmentMenu.getEnvironment() !== '') {
			submitButton.attr('disabled', false);
		} else {
			submitButton.attr('disabled', true);
		}
	});

	for(var i in _menuItems) {
		_form.append(_menuItems[i].getMenuElement());
	}
	
	return {
		getElement : function() {
			return _form;
		}
	};
};

