HAL.singleAlgorithmAnalysisForm = function(options) {
	if ( !(this instanceof HAL.singleAlgorithmAnalysisForm) ) {
	      return new HAL.singleAlgorithmAnalysisForm(options);
	}
	
	var _idPrefix = options.idPrefix == null ? 'newSingleAlgorithmAnalysis' : options.idPrefix;
	
	var _form = $('<form></form>');
	_form.attr('id', _idPrefix+'_form');
	
	var _experimentInformationMenu = new HAL.defaultExperimentInformationMenu({
		idPrefix : _idPrefix,
		metaAlgorithmicPluginCompatibilityFilter : 'SingleAlgorithmAnalysis',
		showPrevious : false,
		showNext : true
	});
	
	var _targetAlgorithmMenu = new HAL.defaultMultipleAlgorithmSelectionMenu({
		idPrefix : _idPrefix,
		enabled : false,
		expanded : false,
		minimumAlgorithms : 1,
		maximumAlgorithms : 1,
		includeParameterisedAlgorithms : true,
		includeParameterlessAlgorithms : true,
		forceParameterlessSelection : true
	});
	
	var _parametersMenu = new HAL.defaultMetaAlgorithmParametersMenu({
		idPrefix : _idPrefix,
		headerContent : 'Analysis procedure parameters',
		enabled : false,
		expanded : false
	});
	
	var _executionEnvironmentMenu = new HAL.defaultExecutionEnvironmentMenu({
		idPrefix : _idPrefix,
		enabled : false,
		expanded : false,
		showNext : false
	});

	_experimentInformationMenu.addOnNext(function() {
		if (_experimentInformationMenu.validate()) {
			_experimentInformationMenu.disable();
			
			_targetAlgorithmMenu.enable();
			_targetAlgorithmMenu.expand();
				
			_targetAlgorithmMenu.update(
				{
					'compatibleWithInstanceDistribution' : _experimentInformationMenu.getInstanceDistribution()	
				},
				function() {},
				function() {}
			);
		}
	});
	
	_targetAlgorithmMenu.addOnPrevious(function() {
		_targetAlgorithmMenu.disable();
		
		_experimentInformationMenu.enable();
		_experimentInformationMenu.expand();
	});
	
	_targetAlgorithmMenu.addOnNext(function() {
		if (_targetAlgorithmMenu.getAlgorithm(0) !== '') {
			_targetAlgorithmMenu.disable();
			_parametersMenu.enable();
			_parametersMenu.expand();
			
			_parametersMenu.update(
				{
					sourceAlgorithm : _experimentInformationMenu.getMetaAlgorithmicProcedure()
				},
				function() {},
				function() {}
			);
		}
	});
	
	_parametersMenu.addOnPrevious(function() {
		_parametersMenu.disable();
		_targetAlgorithmMenu.enable();
		_targetAlgorithmMenu.expand();
	});
	
	_parametersMenu.addOnNext(function() {
		_parametersMenu.disable();
		_executionEnvironmentMenu.enable();
		_executionEnvironmentMenu.expand();
	});
	
	_executionEnvironmentMenu.addOnPrevious(function() {
		_executionEnvironmentMenu.clearMessages();
		_executionEnvironmentMenu.disable();
		
		_parametersMenu.enable();
		_parametersMenu.expand();
	});
	
	var submitButton = $('<input></input>');
	submitButton.attr('type', 'button');
	submitButton.attr('value', 'Submit');
	submitButton.attr('disabled', true);
	submitButton.click(function() {
		_executionEnvironmentMenu.clearMessages();
		submitButton.attr('disabled', true);
		
		var formData = _form.serialize();
		formData = formData+'&hal_httpResponseContentType=json';
		
		$.post('/hal/processFormSnippet/createSingleAlgorithmAnalysis/createSingleAlgorithmAnalysis',
			formData,
			function(response) {
				submitButton.attr('disabled', false);
				
				window.alert('stuff! '+response.successful);
				
				if (response.successful) {
					_executionEnvironmentMenu.addSuccessMessage('Successfully created single algorithm analysis experiment!');
				} else {
					for (var i in response.errorMessages) {
						_executionEnvironmentMenu.addErrorMessage(response.errorMessages[i]);
					}
				}
			},
			'json'
		);
	});
	
	_executionEnvironmentMenu.addNavigationElement(submitButton);
	_executionEnvironmentMenu.addEnvironmentChangeCallback(function() {
		if (_executionEnvironmentMenu.getEnvironment() !== '') {
			submitButton.attr('disabled', false);
		} else {
			submitButton.attr('disabled', true);
		}
	});
	
	// Add the menu to the DOM.
	_form.append(_experimentInformationMenu.getMenuElement());
	_form.append(_targetAlgorithmMenu.getMenuElement());
	_form.append(_parametersMenu.getMenuElement());
	_form.append(_executionEnvironmentMenu.getMenuElement());
	
	return {
		getElement : function() {
			return _form;
		}
	};
};