HAL.singleAlgorithmConfigurationForm = function(options) {
	if ( !(this instanceof HAL.singleAlgorithmConfigurationForm) ) {
	      return new HAL.singleAlgorithmConfigurationForm(options);
	}
	
	var _idPrefix = options.idPrefix == null ? 'newSingleAlgorithmConfiguration' : options.idPrefix;
	
	var _newSingleAlgorithmConfigurationForm = $('<form></form>');
	_newSingleAlgorithmConfigurationForm.attr('id', _idPrefix+'_form');
	
	var _menuItems = new Array(); 
	
	// we had to move specific onclick definitions here
	// because depending on whether we wanted to clone or create
	// we have to inject defaults, and this must happen
	// at the time that we create these functions.
	
	var _newCreateOrCloneMenu = new HAL.createOrCloneMenu({
		idPrefix : _idPrefix,
		enabled : true,
		expanded : true,
		showPrevious : false,
		showNext : false,
		description: 'Define a brand new configuration experiment or clone an already defined experiment'
	}).addNewButtonOnClickHandler(function() {
		
		_newCreateOrCloneMenu.collapse().disable();
		_experimentInformationMenu.enable().expand();
		_newExperimentButtonHandler();
		
	}).addCloneButtonOnClickHandler(function() {
		
		_newCreateOrCloneMenu.collapse().disable();
		_cloneExperimentMenu.enable().expand();
		
	});
	
	_menuItems.push(_newCreateOrCloneMenu);
	
	var _cloneExperimentMenu = new HAL.cloneConfigurationExperimentSelectionMenu({
		idPrefix : _idPrefix,
		//showPrevious : true,
		showPrevious : false,
		showNext : true,
		expanded : false,
		enabled : false
	}).addOnPrevious(function() {
		_cloneExperimentMenu.collapse().disable();
		_newCreateOrCloneMenu.enable().expand();
		
		
	}).addOnNext(function () {
		_cloneExperimentMenu.collapse().disable();
		_experimentInformationMenu.enable().expand().isLoading();
		_cloneExperimentNextButtonHandler();
	});
	
	_menuItems.push(_cloneExperimentMenu);
	
	var _experimentInformationMenu = new HAL.defaultExperimentInformationMenu({
		idPrefix : _idPrefix,
		metaAlgorithmicPluginCompatibilityFilter : 'SingleAlgorithmConfiguration',
		showPrevious : true,
		//showPrevious : false,
		showNext : true,
		expanded : false,
		enabled : false,
		defaults : (!options.defaults) ? undefined : {
			experimentName : 		options.defaults.name(),
			experimentDescription: 	options.defaults.description(),
			metaAlgorithm :			options.defaults.metaAlgorithm(),
			instanceDistribution:	options.defaults.instanceDistribution()
		}
		
	});
	
	_menuItems.push(_experimentInformationMenu);
	
	var _featuresMenu = new HAL.defaultFeatureExtractionAndSelectionMenu({
		idPrefix : _idPrefix,
		enabled : false,
		expanded : false
	});
	
	_menuItems.push(_featuresMenu);
	
	var _targetAlgorithmMenu = new HAL.defaultMultipleAlgorithmSelectionMenu({
		idPrefix : _idPrefix,
		enabled : false,
		expanded : false,
		minimumAlgorithms : 1,
		maximumAlgorithms : 1,
		includeParameterisedAlgorithms : true,
		includeParameterlessAlgorithms : false,
		forceParameterlessSelection : false,
		defaults : (!options.defaults) ? undefined : {
			initialValues : options.defaults.targetAlgorithm(),
		}
	});
	
	_menuItems.push(_targetAlgorithmMenu);
	
	var _parametersMenu = new HAL.defaultMetaAlgorithmParametersMenu({
		idPrefix : _idPrefix,
		headerContent : 'Design procedure parameters',
		enabled : false,
		expanded : false,
		defaults : (!options.defaults) ? undefined : {
			metric : options.defaults.metric(),
			scenario : options.defaults.scenarioOptions(),
			instance : options.defaults.instanceOptions(),
			configuration : options.defaults.configurationOptions(),
		}
	});
	
	_menuItems.push(_parametersMenu);
	
	var _executionEnvironmentMenu = new HAL.defaultExecutionEnvironmentMenu({
		idPrefix : _idPrefix,
		enabled : false,
		expanded : false,
		showNext : false,
		defaults : (!options.defaults) ? undefined : {
			environment : options.defaults.environment(),
		}
	});
	
	_menuItems.push(_executionEnvironmentMenu);
	
	var _newExperimentButtonHandler = function() {
		_experimentInformationMenu.getMenuObject().hidePreviousButton();
		
		registerButtonHandlerFunctions();
	}
	
var _cloneExperimentNextButtonHandler = function() {
		
		_cloneExperimentMenu.isLoading();
		
		$.ajax({
			//url: '/jsp/getAlgorithmRunRequestJSON.jsp',
			url: '/hal/cloneAlgorithm/getAlgorithmRunDetails',
			data : { runId : _cloneExperimentMenu.getSelectorValue() },
			dataType: 'json',
			success: function(data) {
				if (!data.successful) {
					
					for(var i in data.errorMessages) {
						_experimentInformationMenu.addErrorMessage(data.errorMessages[i]);
					}
				} else {
					
					_cloneExperimentMenu.hasLoaded();
					
					var experimentObj = createConfigurationExperimentObject(data.result);
					
					registerButtonHandlerFunctions({
						name : experimentObj.name(),
						description : experimentObj.description(),
						metaAlgorithm : experimentObj.metaAlgorithm(),
						instanceDistribution : experimentObj.instanceDistribution(),
						environment : experimentObj.environment(),
						targetAlgorithms : experimentObj.targetAlgorithm(),
						metric : experimentObj.metric(),
						scenarioOptions : experimentObj.scenarioOptions(),
						instanceOptions : experimentObj.instanceOptions(),
						configurationOptions : experimentObj.configurationOptions(),
						featureExtractors : experimentObj.featureExtractors(),
						
					});
					
				}
			}
		});
	}
	
	var registerButtonHandlerFunctions = function(defaults) {
		if (defaults) {
			//var experimentObj = createConfigurationExperimentObject(data.result);
			
			_experimentInformationMenu.hasLoaded().setDefaults({
				experimentName : defaults.name,
				experimentDescription : defaults.description,
				metaAlgorithm: defaults.metaAlgorithm,
				instanceDistribution: defaults.instanceDistribution,
			});
			
			_executionEnvironmentMenu.setDefaults({
				environment: defaults.environment,
			});
			
			
			submitButton.attr('disabled',false);
		}
		
		_experimentInformationMenu.addOnPrevious(function() {
			if (defaults) {
				_cloneExperimentMenu.enable().expand();
			} else {
				_newCreateOrCloneMneu.enable().expand();
			}
			
			_experimentInformationMenu.collapse().disable();
			
		});
		
		_experimentInformationMenu.addOnNext(function() {
			if (_experimentInformationMenu.validate()) {
				_experimentInformationMenu.disable();
				
				var regex = new RegExp(/.*SMAC.*/);
				
				if (regex.exec(_experimentInformationMenu.getMetaAlgorithmicProcedure()) !== null) {
					_featuresMenu.enable();
					_featuresMenu.expand();
					
					_featuresMenu.update(
						{
							'compatibleWithInstanceDistribution' : _experimentInformationMenu.getInstanceDistribution()
						},
						function() { if (defaults && defaults.featureExtractors) { _featuresMenu.setDefaults({ featureExtractors: defaults.featureExtractors }); } },
						function() {}
					);
				} else {
					_targetAlgorithmMenu.enable();
					_targetAlgorithmMenu.expand();
					
					_targetAlgorithmMenu.update(
						{
							'compatibleWithInstanceDistribution' : _experimentInformationMenu.getInstanceDistribution()	
						},
						function() { if (defaults) { _targetAlgorithmMenu.setDefaults({ targetAlgorithms: defaults.targetAlgorithms }); } },
						function() {}
					);
				}
			}
		});
	
		_featuresMenu.addOnPrevious(function() {
			_featuresMenu.disable();
			
			_experimentInformationMenu.enable();
			_experimentInformationMenu.expand();
		});
		
		_featuresMenu.addOnNext(function() {
			_featuresMenu.disable();
			_targetAlgorithmMenu.enable();
			_targetAlgorithmMenu.expand();
			
			_targetAlgorithmMenu.update(
				{ 'compatibleWithInstanceDistribution' : _experimentInformationMenu.getInstanceDistribution() },
				function() { if (defaults) { _targetAlgorithmMenu.setDefaults({ targetAlgorithms : defaults.targetAlgorithms }); } },
				function() {}
			);
		});
		
		_targetAlgorithmMenu.addOnPrevious(function() {
			_targetAlgorithmMenu.disable();
			
			var regex = new RegExp(/.*SMAC.*/);
			
			if (regex.exec(_experimentInformationMenu.getMetaAlgorithmicProcedure()) !== null) {
				_featuresMenu.enable();
				_featuresMenu.expand();
			} else {
				_experimentInformationMenu.enable();
				_experimentInformationMenu.expand();
			}
		});
		
		_parametersMenu.addOnPrevious(function() {
			_parametersMenu.disable();
			_targetAlgorithmMenu.enable();
			_targetAlgorithmMenu.expand();
		});
		
		_parametersMenu.addOnNext(function() {
			_parametersMenu.disable();
			_executionEnvironmentMenu.enable();
			_executionEnvironmentMenu.expand();
		});
		
		_targetAlgorithmMenu.addOnNext(function() {
			if (_targetAlgorithmMenu.getAlgorithm(0) !== '') {
				_targetAlgorithmMenu.disable();
				_parametersMenu.enable();
				_parametersMenu.expand();
				
				_parametersMenu.update(
					{
						sourceAlgorithm : _experimentInformationMenu.getMetaAlgorithmicProcedure()
					},
					function() { if (defaults) { _parametersMenu.setDefaults({ metric: defaults.metric }); } },
					function() {},
					(!defaults) ? undefined : {
						scenario : defaults.scenarioOptions,
						instance : defaults.instanceOptions,
						configuration : defaults.configurationOptions,
					}
				);
			}
		});
		
		_executionEnvironmentMenu.addOnPrevious(function() {
			_executionEnvironmentMenu.clearMessages();
			_executionEnvironmentMenu.disable();
			
			_parametersMenu.enable();
			_parametersMenu.expand();
		});
		
		
	}

	var submitButton = $('<input></input>');
	submitButton.attr('type', 'button');
	submitButton.attr('value', 'Submit');
	submitButton.attr('disabled', (options.defaults) ? false : true);
	submitButton.click(function() {
		_executionEnvironmentMenu.clearMessages();
		submitButton.attr('disabled', true);
		
		var formData = _newSingleAlgorithmConfigurationForm.serialize();
		formData = formData+'&hal_httpResponseContentType=json';
		
		$.post('/hal/processFormSnippet/createSingleAlgorithmConfiguration/createSingleAlgorithmConfiguration',
			formData,
			function(response) {
				submitButton.attr('disabled', false);
				
				if (response.successful) {
					_executionEnvironmentMenu.addSuccessMessage('Successfully created single algorithm configuration experiment!');
					_executionEnvironmentMenu.addSuccessMessage("Redirecting back to HAL's homepage");
					setTimeout(function() {
						window.location="/hal";
					}, 2000);
				} else {
					for (var i in response.errorMessages) {
						_executionEnvironmentMenu.addErrorMessage(response.errorMessages[i]);
					}
				}
			},
			'json'
		);
	});
	
	_executionEnvironmentMenu.addNavigationElement(submitButton);
	_executionEnvironmentMenu.addEnvironmentChangeCallback(function() {
		if (_executionEnvironmentMenu.getEnvironment() !== '') {
			submitButton.attr('disabled', false);
		} else {
			submitButton.attr('disabled', true);
		}
	});
	
	// Add the menu to the DOM.
	for(var i in _menuItems) {
		_newSingleAlgorithmConfigurationForm.append(_menuItems[i].getMenuElement());
	}
	
	return {
		getElement : function() {
			return _newSingleAlgorithmConfigurationForm;
		},
	};
};