package ca.ubc.cs.beta.activeconfigurator.matlab.predictivemodels;

import com.mathworks.toolbox.javabuilder.pooling.Poolable;
import java.util.List;
import java.rmi.Remote;
import java.rmi.RemoteException;

/**
 * The <code>PredictiveModel</code class provides a Java interface to the
 * M-functions from the files:
 * <pre>
 * /Users/chrisnell/Documents/workspace/matlab/PredictivePerfModels-Code/modelPr
 * edictForJava.m
 * /Users/chrisnell/Documents/workspace/matlab/PredictivePerfModels-Code/read_mo
 * del_from_file.m
 * </pre>
 * The {@link #dispose} method <b>must</b> be called on a
 * <code>PredictiveModel</code> instance when it is no longer needed to ensure
 * that native resources allocated by this class are properly freed.
 * @version 0.0
 */
public interface PredictiveModelRemote extends Poolable
{
  /**
   * Provides the standard interface for calling the
   * <code>modelPredictForJava</code> M-function with 3 input arguments. 
   * Input arguments may be passed as sub-classes of
   * <code>com.mathworks.toolbox.javabuilder.MWArray</code>, or as arrays of
   * any supported Java type. Arguments passed as Java types are converted to
   * MATLAB arrays according to default conversion rules.
   * @param nargout Number of outputs to return.
   * @param rhs The inputs to the M function.
   * @return Array of length nargout containing the function outputs. Outputs
   * are returned as sub-classes of
   * <code>com.mathworks.toolbox.javabuilder.MWArray</code>. Each output array
   * should be freed by calling its <code>dispose()</code> method.
   * @throws MWException An error has occured during the function call.
   */
  public Object[] modelPredictForJava(int nargout, Object... rhs) throws RemoteException;
  
  /**
   * Provides the standard interface for calling the
   * <code>read_model_from_file</code> M-function with 1 input argument. 
   * Input arguments may be passed as sub-classes of
   * <code>com.mathworks.toolbox.javabuilder.MWArray</code>, or as arrays of
   * any supported Java type. Arguments passed as Java types are converted to
   * MATLAB arrays according to default conversion rules.
   * @param nargout Number of outputs to return.
   * @param rhs The inputs to the M function.
   * @return Array of length nargout containing the function outputs. Outputs
   * are returned as sub-classes of
   * <code>com.mathworks.toolbox.javabuilder.MWArray</code>. Each output array
   * should be freed by calling its <code>dispose()</code> method.
   * @throws MWException An error has occured during the function call.
   */
  public Object[] read_model_from_file(int nargout, Object... rhs) throws RemoteException;
  
  /** Frees native resources associated with the remote server object */
  void dispose() throws RemoteException;
}
