function do_ehm_experiments(X,y,featureNames,domain)
% Given data X and y, experiment with different models.

exp1 = 0;
exp2 = 0;
exp3 = 1;

fprintf(strcat(['EHM experiments for domain ', domain, '\n']));

%% Experiment 1: Evaluate performance by cross validation.
if exp1
    fprintf(strcat(['EHM experiment 1 for domain ', domain, '\n']));
    k = 10; % k-fold crossval.
    options_vec = {get_rf_default_options,get_gp_default_options,get_lr_default_options};
    for model_idx=1:length(options_vec)
        fprintf(strcat(['  Model ', num2str(model_idx), ' ... ']));
        [rmses(model_idx,:), lls(model_idx,:), ccs(model_idx,:), learnTimes(model_idx,:), predTimes(model_idx,:)] = repeated_simple_model_perf(X, y, 1000, options_vec{model_idx},10);
%         [rmses(model_idx,:), lls(model_idx,:), ccs(model_idx,:), learnTimes(model_idx,:), predTimes(model_idx,:)] = crossval_model_perf(X, y, k, options_vec{model_idx}, 1, domain);
    end
    modelNames=cell(length(options_vec),1);
    for i=1:length(modelNames)
        modelNames{i,1} = options_vec{i}.modelType;
    end
    stats_names = {'RMSE', 'CC', 'LL', 'Time to learn'};
    header_output(modelNames, stats_names);
    fprintf(strcat(domain, ' & '));
    n_output(mean(rmses,2),1);
    n_output(mean(ccs,2),1);
    n_output(mean(lls,2),0);
    n_output(mean(learnTimes,2),1);
    fprintf('\n');    
end


%% Experiment 2: scaling performance with # data points
if exp2
    fprintf(strcat(['EHM experiment 2 for domain ', domain, '\n']));
    options_vec = {get_rf_default_options,get_gp_default_options,get_lr_default_options};
    modelNames = {'RF', 'GP', 'RR'};

    numTrains = [2,4,8,16,32,64,128,256,512,1024];
    for i=1:length(numTrains)
        numTrain = numTrains(i);
        fprintf(strcat([' numTrain = ', num2str(numTrain), '\n']));

        for model_idx = 1:length(options_vec)
            fprintf(strcat(['  Model ', num2str(model_idx), ' ... ']));
            [rmses, lls, ccs, timesToLearn, timesToPredict] = repeated_simple_model_perf(X, y, numTrain, options_vec{model_idx},10);

            rmse_avg(i,model_idx) = mean(rmses);
            rmse_std(i,model_idx) = std(rmses);
            lls_avg(i,model_idx) = mean(lls);
            lls_std(i,model_idx) = std(lls);
            cc_avg(i,model_idx) = mean(ccs);
            cc_std(i,model_idx) = std(ccs);
            timesToLearn_avg(i,model_idx) = mean(timesToLearn);
            timesToLearn_std(i,model_idx) = std(timesToLearn);
            timesToPredict_avg(i,model_idx) = mean(timesToPredict);
            timesToPredict_std(i,model_idx) = std(timesToPredict);
        end
    end
    stats_names = {'RMSE, N=128', 'RMSE, N=1024', 'Time, N=128', 'Time, N=1024'};
    header_output(modelNames, stats_names);
    fprintf(strcat(domain, ' & '));
    n_output(rmse_avg(7,:),1);
    n_output(rmse_avg(10,:),1);
    n_output(timesToLearn_avg(7,:),1);
    n_output(timesToLearn_avg(10,:),1);
    fprintf('\n');    
    
    plot_Scaling_N(numTrains, cc_avg, cc_std, options_vec, 'Correlation coefficient', modelNames, domain, 'NorthWest');
    plot_Scaling_N(numTrains, rmse_avg, rmse_std, options_vec, 'RMSE', modelNames, domain, 'NorthEast');
    plot_Scaling_N(numTrains, lls_avg, lls_std, options_vec, 'Test Data Log likelihood', modelNames, domain);
    plot_Scaling_N(numTrains, timesToLearn_avg, timesToLearn_std, options_vec, 'Learn Time', modelNames, domain, 'NorthWest', 1);
    plot_Scaling_N(numTrains, timesToPredict_avg, timesToPredict_std, options_vec, 'Prediction Time', modelNames, domain, 'NorthWest', 1);
end

%% Experiment 3: scaling performance with #trees, size of active set, and #features.
if exp3
    fprintf(strcat(['EHM experiment 3 for domain ', domain, '\n']));

    rf_options = get_rf_default_options;
    numScal{1} = [1,2,4,8,16,32,64,128,256,512,1024];
    fprintf(strcat([' Model type 1/4: RF ...\n']));
    for i=1:length(numScal{1})
        fprintf(strcat(['  Version ', num2str(i), '/', num2str(length(numScal{1})), ' ...\n']));
        rf_options.nSub = numScal{1}(i);
        [rmses{1}{i}, lls{1}{i}, ccs{1}{i}, timesToLearn{1}{i}, timesToPredict{1}{i}] = repeated_simple_model_perf(X, y, 1000, rf_options, 10);
    end

    gp_options = get_gp_default_options;
    numScal{2} = [1,2,4,8,16,32,64,128,256,512,1024];
    fprintf(strcat([' Model type 2/4: GP ...\n']));
    for i=1:length(numScal{2})
        fprintf(strcat(['  Version ', num2str(i), '/', num2str(length(numScal{2})), ' ...\n']));
        gp_options.ppSize = numScal{2}(i);
        gp_options.trainSubSize = numScal{2}(i);
        [rmses{2}{i}, lls{2}{i}, ccs{2}{i}, timesToLearn{2}{i}, timesToPredict{2}{i}] = repeated_simple_model_perf(X, y, 1000, gp_options, 10);
    end

    lr_options = get_lr_default_options;
    %options_lr.linearSize = 30;
    numScal{3} = [1,2,4,8,16,32,64]%,128,256];
    fprintf(strcat([' Model type 3/4: Ridge regression ...\n']));
    for i=1:length(numScal{3})
        fprintf(strcat(['  Version ', num2str(i), '/', num2str(length(numScal{3})), ' ...\n']));
        lr_options.maxModelSize = numScal{3}(i);
        [rmses{3}{i}, lls{3}{i}, ccs{3}{i}, timesToLearn{3}{i}, timesToPredict{3}{i}] = repeated_simple_model_perf(X, y, 1000, lr_options, 10);
    end

    numScal{4} = [1];
    fprintf(strcat([' Model type 4/4: RegTree ...\n']));
    regtree_options = get_regtree_default_options;
    [rmses{4}{1}, lls{4}{1}, ccs{4}{1}, timesToLearn{4}{1}, timesToPredict{4}{1}] = repeated_simple_model_perf(X, y, 1000, regtree_options, 10);

    modelTimesAvg = {};
    for model_idx=1:4
        for i=1:length(numScal{model_idx})
            rmse_avg{model_idx}{i} = mean(rmses{model_idx}{i});
            rmse_std{model_idx}{i} = std(rmses{model_idx}{i});
            ll_avg{model_idx}{i} = mean(lls{model_idx}{i});
            ll_std{model_idx}{i} = std(lls{model_idx}{i});
            cc_avg{model_idx}{i} = mean(ccs{model_idx}{i});
            cc_std{model_idx}{i} = std(ccs{model_idx}{i});
            timesToLearn_avg{model_idx}{i} = mean(timesToLearn{model_idx}{i});
            timesToLearn_std{model_idx}{i} = std(timesToLearn{model_idx}{i});
            timesToPredict_avg{model_idx}{i} = mean(timesToPredict{model_idx}{i});
            timesToPredict_std{model_idx}{i} = std(timesToPredict{model_idx}{i});
            modelTimesAvg{model_idx}{i} = timesToLearn_avg{model_idx}{i} + timesToPredict_avg{model_idx}{i};
        end
    end

    model_names = {'RF', 'PP', 'RR', 'RegTree'};
    namesNumScal = {'#trees', 'dummy', 'Size of active set', '#features'};
    plot_Scaling_Model_Compl(numScal, namesNumScal, cc_avg, cc_std, modelTimesAvg, 'Correlation coefficient', model_names, domain, 'East');
    plot_Scaling_Model_Compl(numScal, namesNumScal, rmse_avg, rmse_std, modelTimesAvg, 'RMSE', model_names, domain, 'NorthEast');
    plot_Scaling_Model_Compl(numScal, namesNumScal, ll_avg, ll_std, modelTimesAvg, 'Test Data Log likelihood', model_names, domain);
    plot_Scaling_Model_Compl(numScal, namesNumScal, timesToLearn_avg, timesToLearn_std, modelTimesAvg, 'Learn Time', model_names, domain, 'NorthWest', 1);
    plot_Scaling_Model_Compl(numScal, namesNumScal, timesToPredict_avg, timesToPredict_std, modelTimesAvg, 'Prediction Time', model_names, domain, 'NorthWest', 1);
end
