function simple_prediction_experiment
% Carries out the experiment for simple empirical hardness models
% (i.e., only features vary).

if isunix
    dir = '/ubc/cs/project/arrow/hutter/model_paper/';
else
    dir = 'Z:\arrowspace\model_paper\';
end

mip_benchmark_files = {};
sat_benchmark_files = {};
mip_benchmark_files{end+1,1} = 'cplex-corlat.csv';
mip_benchmark_files{end+1,1} = 'gurobi-corlat.csv';
mip_benchmark_files{end+1,1} = 'scip-corlat.csv';

mip_benchmark_files{end+1,1} = 'cplex-bigmix.csv';
mip_benchmark_files{end+1,1} = 'gurobi-bigmix.csv';
mip_benchmark_files{end+1,1} = 'scip-bigmix.csv';

benchmark_files = [mip_benchmark_files; sat_benchmark_files];


for i=1:length(benchmark_files)
    benchmark_file = strcat(dir, benchmark_files{i});
    
    %=== Get X and y from that file.
    savedXy_file = strcat(benchmark_file, '-saved')
    if exist(savedXy_file, 'file')
        data = csvread(savedXy_file);
        X = data(:,1:end-1);
        y = data(:,end);
        featureNames = get_mip_feature_names;
    else
        y = csvread(benchmark_file,0,1); % runtimes

        instance_names = textread(benchmark_file,'%s%*[^\n]', 'bufsize', 10000);
        for j=1:length(instance_names)
            instance_names{j} = instance_names{j}(1:end-1); % remove the trailing comma
        end

        %=== Get features for instance names from database.
        if mysql('status')
            startUpDB
        end
        mysql('use MODEL_DB');
        global feature_table;
        feature_table = 'MIP_FEATURES';
        inst_ids = get_inst_id(instance_names);
        if i<= length(mip_benchmark_files)
            mip = 1;
        else
            mip = 0;
        end   
        [X, featureNames] = getFeaturesForID(mip, inst_ids);
        csvwrite(savedXy_file, [X, y]);

%         feature_file = strcat(dir, 'MIP_feats.csv');
% %        feature_file = strcat(dir, 'bigmix_feats.csv');
%         featureNames = {};
%         X = readFeatures(instance_names, feature_file);

    end

    domain_name = regexprep(benchmark_files{i}, '\.', '-');
    do_ehm_experiments(X,y,featureNames,domain_name);
%    close all
end


function mip_feature_names = get_mip_feature_names
mip_feature_names = {'n_constraints'; 'n_variables'; 'nzcnt'; 'probtype'; 'num_q_constr'; 'num_quad'; 'num_qpnz'; 'vcg_var_deg_mean'; 'vcg_var_deg_std'; 'vcg_var_deg_min'; 'vcg_var_deg_max'; 'vcg_con_deg_mean'; 'vcg_con_deg_std'; 'vcg_con_deg_min'; 'vcg_con_deg_max'; 'prices_stddev'; 'price_per_number_stddev'; 'price_per_sqrt_number_stddev'; 'perc_cont_var'; 'support_size_mean'; 'support_size_std'; 'a_ij_magnitude_mean'; 'a_ij_magnitude_std'; 'a_ij_varcoef_avg'; 'a_ij_varcoef_std'; 'perc_unbounded_discrete'; 'edge_density'; 'vg_deg_std'; 'vg_deg_max'; 'vg_deg_min'; 'vg_deg_q0_25'; 'vg_deg_med'; 'vg_deg_q0_75'; 'clust_coef'; 'deviation'; 'lp_avg'; 'lp_l2_avg'; 'lp_linf'; 'lp_objval'};
%clean: [1:26,36:39]

function X = readFeatures(instance_names, feature_file)
X_file = csvread(feature_file, 0, 1);
instance_names_file = textread(feature_file,'%s%*[^\n]', 'bufsize', 200000);

for i=1:length(instance_names)
    for j=1:length(instance_names_file)
        succ=0;
        if strcmp(instance_names{i}, instance_names_file{j})
            X(i,:) = X_file(j,:);
            succ=1;
            break
        end
    end
    if(succ==0)
        instance_names{i}
        error(strcat(['no instance features not defined for instance ', instance_names{i}, ' in file ', feature_file]));
    end
end